#
#           Multiple Linear Regression Analysis
#                        Yasuharu Okamoto, 2018
#
from readdataMR import *
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as scst

print('Input data type...')
print('Data is set in the list RawData -> 1')
print('Data is set in the text file    -> 2')
print('Data is set in the csv file     -> 3')
ck = input('\nYour choice = ')
try:
    if ck == '1':
        RawData, f_out, f_out_nm = ReadData_lst()
    elif ck == '2':
        RawData, f_out, f_out_nm = ReadData_txt()
    elif ck == '3':
        RawData, f_out, f_out_nm = ReadData_csv()
    else:
        print('\nInvalid choice...')
        raise Exception()

    f_out.write('Data...\n')
    for v in RawData:
        f_out.write('{0}\n'.format(v))

    VarNames = RawData[0]
    ID = []
    y = []                          #   Dependent variable
    X = []                          #   Independent variables
    for d in RawData[1:]:
        ID.append(d[0])
        y.append([d[1]])
        X.append([1] + d[2:])
    
    y = np.array(y)
    X = np.array(X)

    
    b = np.linalg.inv(X.transpose() @ X) @ X.transpose() @ y
    print('\nb = ', b)

    y_est = X @ b                   #   predicted values of of

    r, p = scst.pearsonr(y, y_est)  #   Correlation coefficient
    r = r[0]
    print('R = {0:.3}'.format(r))
    R2 = r ** 2                     #   Coefficient of determination
    print('R2 = {0:.3}'.format(R2))
    #
    #           Plotting (y_est, y)
    #
    plt.plot(y_est, y, 'bo')
    
    min_x = np.amin(y_est)
    max_x = np.amax(y_est)
    plt.plot([min_x, max_x],[min_x, max_x], 'b-')
    #
    #           Displaying Labels
    #
    disp_x = (max_x - min_x) * 0.01
    disp_y = (np.max(y) - np.min(y)) * 0.01
    for v_est, v, name in zip(y_est, y, ID):
        plt.text(v_est + disp_x, v + disp_y, name)
    plt.xlabel('predicted y')
    plt.ylabel('y')
    plt.title('Multiple Linear Regression\n' +
              'R = {0:.3}   $R^2$ = {1:.3}'.format(r, R2))

    plt.show()
    #
    #           Writing the regression coefficients
    #
    for i in range(X.shape[1]):
        if i == 0:
            f_out.write('\nb0<Constant>:\n     {0:.5}\n'.format(b[0][0]))
        else:
            f_out.write('b{0}<{1}>:\n     {2:.5}\n'.format(i, VarNames[i + 1], b[i][0]))
    
    f_out.write('\nR = {0:.3}    R^2 = {1:.3}\n'.format(r, R2))
    
    
    f_out.close()
    print('{0} was saved.'.format(f_out_nm))

except Exception as e:
    print('\nException...{}\n'.format(e))


